{% extends "base.html" %}
{% load bootstrap %}

{% block content %}
    <H1>Click the following to upload spreadsheet</H1>
    <a href="upload/">Upload</a>
    <br/>
    <form action="" method="post" enctype="multipart/form-data" class="form-inline">
        {% csrf_token %}
        <legend>Target file:</legend>
        {{ form|bootstrap}}<br/><br/>
        <div class="form-group">
            <!--<input type="submit" />-->
            <button type="submit" class="btn btn-primary">Submit</button>
        </div>
    </form>
    <pre lang="python">{{codes}}</pre>

{% endblock %}
