{% extends "base.html" %}
{% load bootstrap %}

{% block content %}
    <H1>Click the following to create model for spreadsheet</H1>
    <a href="../">Create model</a>
    <br/>
    <form action="" enctype="multipart/form-data" method="post" class="form-inline">
        <legend>file:</legend>
        {{ file_form|bootstrap}}<br/><br/>
        <div class="form-group">
            <!--<input type="submit" />-->
            <button type="submit" class="btn btn-primary">Submit</button>
        </div>
    </form>
{% endblock %}
